% Performs the prinsipal component analysis and adjusts the Xi matrix

X = spot(:,1:24);
[coef score lambda] = princomp(X);      % Performs PCA

coef = coef(:,1:n_pr);                  % Reduced coefficient matrix
PC = coef'*X';                          % Principal component scenarios

var_cap = sum(lambda(1:n_pr))/sum(lambda);  % share of variance in spot 
                                            % prices captured by the PC
r_i = zeros(24,1);
ppoints = cell(24,1);

for i = 1:24
    xi_min = min(xi(:,2:25));       % min spot prices
    xi_max = max(xi(:,2:25));       % max spot prices
    ppoints{i} = pricepoints; 
    ok = false;
    while ~ok && length(ppoints{i}) >= 1
     	if xi_min(i) >= ppoints{i}(1)  % rho_min must be < first price point
          	error('Your pricepoints are badly defined');
      	elseif xi_max(i) <= ppoints{i}(end) % rho_max must be > last price point
          	error('Your pricepoints are badly defined');
        else
         	ok = true;
        end
    end
	r_i(i) = length(ppoints{i})+1;
 	if r_i(i) == 1
     	error('Your pricepoints are badly defined');
  	end
end
if min(r_i) ~= max(r_i)
    error('Your pricepoints are badly defined');
end

pMat = [];
for i = 1:length(pricepoints)
    pMat = [pMat pricepoints(i)*ones(24,1)];
end

breaks = coef'*pMat;
            % Breakpoints in principal components
for i = 1:n_pr
    if sum(sort(breaks(i,:)) == breaks(i,:)) <= length(pricepoints) - 1
        error('Too many pricepoints');
    end
end

% k-values original data
ksL(1) = ks(1);                 % Lifted k^s
ksL(2) = ksL(1) + sum(r_i);
for s = 3:S
    ksL(s) = ksL(s-1) + k_s(s);
end
kL = ksL(end);
% k-values PC-data
ksP(1) = ks(1);
ksP(2) = ksP(1) + n_pr*r_i(1);
for s = 3:S
    ksP(s) = ksP(s-1) + k_s(s);
end
kP = ksP(end);

xiP = zeros(n_scen, kP);        % lifted uncertain PC parameters
xiP(:,1) = xi(:,1);             % xi_1

% Break xi into piecewise xi
PC_min = min(PC')';
PC_max = max(PC')';
i_P = 1;
for i = 1:n_pr
    i_P = i_P + 1;
    xiP(:,i_P) = min(PC(i,:)', breaks(i,1));
    for j = 2:r_i(i)-1
        i_P = i_P + 1;
        xiP(:,i_P) = max(min(PC(i,:)',breaks(i,j))-breaks(i,j-1),0);
    end
    i_P = i_P + 1;
    xiP(:,i_P) = max(PC(i,:)'-breaks(i,r_i(i)-1),0);
end
xiP(:,ksP(2)+1:end) = xi(:,ks(2)+1:end);

if length(find((min(xiP)-max(xiP))==0)) >= 2   
    % Test whether some lifted parameters are deterministic
    error('At least one pricepoint is redundant for one hour');
end

xi      = xiP;
ks      = ksP;
k       = kP;
k_s(2)  = n_pr*r_i(1);

W = [[1;-1] zeros(2,k-1)];  % bounds on xi_1 := 1
h = [1;-1];

% Generate W_i matrices:
for i = 1:n_pr
    W_i = zeros(r_i(i)+1,r_i(i));
    W_i1 = zeros(r_i(i)+1, 1);          % First row
    W_i1(1:2) = [breaks(i,1)/(breaks(i,1)-PC_min(i));...
                 -PC_min(i)/(breaks(i,1)-PC_min(i))];
    W_i(1:2,1) = [-1/(breaks(i,1)-PC_min(i));...
                  1/(breaks(i,1)-PC_min(i))];
    for j = 2:r_i(i)-1
        W_i(j:j+1,j) = [-1/(breaks(i,j)-breaks(i,j-1));...
                        1/(breaks(i,j)-breaks(i,j-1))];
    end
    W_i(end-1:end,end) = [-1/(PC_max(i)-breaks(i,end));....
                          1/(PC_max(i)-breaks(i,end))];
    W_add = [W_i1 zeros(r_i(i)+1, k-1)];
    if i == 1
        W_add(:,2:r_i(i)+1) = W_i;
    else
        W_add(:, (sum(r_i(1:i-1))+2):(sum(r_i(1:i))+1)) = W_i;
    end
    W = [W;W_add];
    h = [h;zeros(r_i(i)+1,1)]; 
end

% Upper and lower bounds on xi for s > 2:
W_add = zeros((k-ks(2))*2,k);
W_add(:,ks(2)+1:end) = [eye(k-ks(2));-eye(k-ks(2))];
W = [W;W_add];
h = [h;min(xi(:,ks(2)+1:end))'; -max(xi(:,ks(2)+1:end))'];







    
