function [vars] = spotvar(data, training_period)
% vars = spotvar(data, training_period)
%
% Generates spot forecast error variances for each hour during traing 
% period (in weeks). Seperates into hours (rows) for weekdays (column 1) 
% and weekends (column 2)
%
% Written by Anders Lund Eriksrud, October - November 2012

price       = data(:,1);
days        = data(:,2);
hour        = data(:,3);
n           = size(price,1);

weekdays    = zeros(training_period*5, 24);   % predefine array for weekday
weekends    = zeros(training_period*2, 24);   % predefine array for weekend

pricecounter = n-training_period*24*7+1;      % start at beginning hour           
wdayCounter = 1;
wendCounter = 1;

for day = 1 : training_period*7         % loop thru the last 8 weeks
    if days(pricecounter) <= 5          % weekday
        for i = 1:24                    % loop thru hours in day
            weekdays(wdayCounter, hour(pricecounter) + 1) = ...
                price(pricecounter);
            pricecounter = pricecounter + 1;
        end
        wdayCounter = wdayCounter + 1;
    else                                % weekend
        for i = 1:24                    % loop thru hours in day
            weekends(wendCounter, hour(pricecounter) + 1) = ...
                price(pricecounter);
            pricecounter = pricecounter + 1;
        end
        wendCounter = wendCounter + 1; 
    end
end

vars        = zeros(24,2);
vars(:,1)   = var(weekdays);
vars(:,2)   = var(weekends);

end